STATIC_KEY = "static_key"
GLOBAL_KEY = "global_key"
GLOBAL_LSN = "global_lsn"
VAL = "value"
REF = "ref"
LBRACKET = "LBRACKET"
RBRACKET = "RBRACKET"
EOF_NAME = "EOF"
METADATA = "metadata"
SPARSE_CHECKPOINTS = "sparse_checkpoints"
ITERATIONS_COUNT = "iterations_count"
PKL_SFX = ".pkl"
